@@ -21,6 +21,12 @@
 
 #include "common.h"
 
+//little build hack for vs2015 (due to static libs linked in by lua/luaperks)
+//could probably rebuild those libs, but this is easier
+#if _MSC_VER >= 1900
+extern "C" { FILE __iob_func[3] = { *stdin,*stdout,*stderr }; }
+#endif
+
 // I like hacks.
 #define uint8 __UNO492032
 #include <winsock.h>
@@ -106,6 +112,15 @@ extern bool taseditorEnableAcceleratorKeys;
  #define __COMPILER__STRING__ "unknown"
 #endif
 
+// 64-bit build requires manifest to use common controls 6 (style adapts to windows version)
+#pragma comment(linker, \
+    "\"/manifestdependency:type='win32' "\
+    "name='Microsoft.Windows.Common-Controls' "\
+    "version='6.0.0.0' "\
+    "processorArchitecture='*' "\
+    "publicKeyToken='6595b64144ccf1df' "\
+    "language='*'\"")
+
 // External functions
 extern std::string cfgFile;		//Contains the filename of the config file used.
 extern bool turbo;				//Is game in turbo mode?
@@ -126,7 +141,7 @@ int eoptions = EO_BGRUN | EO_FORCEISCALE | EO_BESTFIT | EO_BGCOLOR | EO_SQUAREPI
 
 //global variables
 int soundoptions = SO_SECONDARY | SO_GFOCUS;
-int soundrate = 44100;
+int soundrate = 48000;
 int soundbuftime = 50;
 int soundquality = 1;
 
@@ -146,14 +161,19 @@ double tvAspectX = TV_ASPECT_DEFAULT_X, tvAspectY = TV_ASPECT_DEFAULT_Y;
 int genie = 0;
 int pal_emulation = 0;
 int pal_setting_specified = 0;
+int dendy = 0;
+int dendy_setting_specified = 0;
+bool swapDuty = 0; // some Famicom and NES clones had duty cycle bits swapped
 int ntsccol = 0, ntsctint, ntschue;
 std::string BaseDirectory;
 int PauseAfterLoad;
 unsigned int skippy = 0;  //Frame skip
 int frameSkipCounter = 0; //Counter for managing frame skip
 // Contains the names of the overridden standard directories
-// in the order roms, nonvol, states, fdsrom, snaps, cheats, movies, memwatch, macro, input presets, lua scripts, base
-char *directory_names[14] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+// in the order roms, nonvol, states, fdsrom, snaps, cheats, movies, memwatch, basic bot, macro, input presets, lua scripts, avi, base
+char *directory_names[14] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+int edit_id[14] = { EDIT_ROM, EDIT_BATTERY, EDIT_STATE, EDIT_FDSBIOS, EDIT_SCREENSHOT, EDIT_CHEAT, EDIT_MOVIE, EDIT_MEMWATCH, EDIT_BOT, EDIT_MACRO, EDIT_PRESET, EDIT_LUA, EDIT_AVI, EDIT_ROOT };
+int browse_btn_id[14] = {BUTTON_ROM, BUTTON_BATTERY, BUTTON_STATE, BUTTON_FDSBIOS, BUTTON_SCREENSHOT, BUTTON_CHEAT, BUTTON_MOVIE, BUTTON_MEMWATCH, BUTTON_BOT, BUTTON_MACRO, BUTTON_PRESET, BUTTON_LUA, BUTTON_AVI, BUTTON_ROOT };
 std::string cfgFile = "fceux.cfg";
 //Handle of the main window.
 HWND hAppWnd = 0;
@@ -266,7 +286,7 @@ void DefaultDirectoryWalker(void (*callback)(const char*))
 			sprintf(
 				TempArray,
 				"%s\\%s",
-				directory_names[NUMBER_OF_DEFAULT_DIRECTORIES] ? directory_names[NUMBER_OF_DEFAULT_DIRECTORIES] : BaseDirectory.c_str(),
+				directory_names[NUMBER_OF_DEFAULT_DIRECTORIES - 1] ? directory_names[NUMBER_OF_DEFAULT_DIRECTORIES - 1] : BaseDirectory.c_str(),
 				default_directory_names[curr_dir]
 			);
 
@@ -307,54 +327,118 @@ int BlockingCheck()
 	MSG msg;
 	moocow = 1;
 
-	while( PeekMessage( &msg, 0, 0, 0, PM_NOREMOVE ) )
+	while(PeekMessage(&msg, 0, 0, 0, PM_NOREMOVE))
 	{
-		if( GetMessage( &msg, 0,  0, 0)>0 )
+		if(GetMessage(&msg, 0, 0, 0) > 0)
 		{
 			//other accelerator capable dialogs could be added here
-			extern HWND hwndMemWatch;
-
 			int handled = 0;
 
-			if(hCheat)
-				if(IsChild(hCheat, msg.hwnd))
-					handled = IsDialogMessage(hCheat, &msg);
-			if(!handled && hMemFind)
-			{
-				if(IsChild(hMemFind, msg.hwnd))
-					handled = IsDialogMessage(hMemFind, &msg);
-			}
+			// Cheat console
+			if(hCheat && IsChild(hCheat, msg.hwnd))
+				handled = IsDialogMessage(hCheat, &msg);
+
+			// Hex Editor -> Find
+			if(!handled && hMemFind && IsChild(hMemFind, msg.hwnd))
+				handled = IsDialogMessage(hMemFind, &msg);
+
+			// Memory Watch
+			extern HWND hwndMemWatch;
 			if(!handled && hwndMemWatch)
 			{
-				if(IsChild(hwndMemWatch,msg.hwnd))
-					handled = TranslateAccelerator(hwndMemWatch,fceu_hAccel,&msg);
+				if(IsChild(hwndMemWatch, msg.hwnd))
+					handled = TranslateAccelerator(hwndMemWatch, fceu_hAccel, &msg);
 				if(!handled)
 					handled = IsDialogMessage(hwndMemWatch,&msg);
 			}
-			if(!handled && RamSearchHWnd)
-			{
+
+			// RAM Search
+			if(!handled && RamSearchHWnd && IsChild(RamSearchHWnd, msg.hwnd))
 				handled = IsDialogMessage(RamSearchHWnd, &msg);
-			}
+
+			// RAM_Watch
 			if(!handled && RamWatchHWnd)
-			{
-				if(IsDialogMessage(RamWatchHWnd, &msg))
-				{
+				if(handled = IsDialogMessage(RamWatchHWnd, &msg))
 					if(msg.message == WM_KEYDOWN) // send keydown messages to the dialog (for accelerators, and also needed for the Alt key to work)
 						SendMessage(RamWatchHWnd, msg.message, msg.wParam, msg.lParam);
-					handled = true;
+
+			// TAS Editor
+			if(!handled && taseditorWindow.hwndTASEditor)
+			{
+				if(taseditorEnableAcceleratorKeys)
+					if(IsChild(taseditorWindow.hwndTASEditor, msg.hwnd))
+						handled = TranslateAccelerator(taseditorWindow.hwndTASEditor, fceu_hAccel, &msg);
+				if(!handled){
+					handled = IsDialogMessage(taseditorWindow.hwndTASEditor, &msg);
 				}
 			}
 
-			if(!handled && taseditorWindow.hwndTASEditor && taseditorEnableAcceleratorKeys)
-			{
-				if(IsChild(taseditorWindow.hwndTASEditor, msg.hwnd))
-					handled = TranslateAccelerator(taseditorWindow.hwndTASEditor, fceu_hAccel, &msg);
-			}
-			if(!handled && taseditorWindow.hwndFindNote)
-			{
-				if(IsChild(taseditorWindow.hwndFindNote, msg.hwnd))
-					handled = IsDialogMessage(taseditorWindow.hwndFindNote, &msg);
-			}
+			// TAS Editor -> Find Node
+			if(!handled && taseditorWindow.hwndFindNote && IsChild(taseditorWindow.hwndFindNote, msg.hwnd))
+				handled = IsDialogMessage(taseditorWindow.hwndFindNote, &msg);
+
+			// Sound Config
+			extern HWND uug;
+			if(!handled && uug && IsChild(uug, msg.hwnd))
+				handled = IsDialogMessage(uug, &msg);
+
+			// Palette Config
+			extern HWND hWndPal;
+			if(!handled && hWndPal && IsChild(hWndPal, msg.hwnd))
+				handled = IsDialogMessage(hWndPal, &msg);
+
+			// Code/Data Logger
+			if(!handled && hCDLogger && IsChild(hCDLogger, msg.hwnd))
+				handled = IsDialogMessage(hCDLogger, &msg);
+
+			// Trace Logger
+			if(!handled && hTracer && IsChild(hTracer, msg.hwnd))
+				handled = IsDialogMessage(hTracer, &msg);
+
+			// Game Genie Encoder/Decoder
+			extern HWND hGGConv;
+			if(!handled && hGGConv && IsChild(hGGConv, msg.hwnd))
+				handled = IsDialogMessage(hGGConv, &msg);
+
+			// Debugger
+			if(!handled && hDebug && IsChild(hDebug, msg.hwnd))
+				handled = IsDialogMessage(hDebug, &msg);
+
+			// PPU Viewer
+			extern HWND hPPUView;
+			if(!handled && hPPUView && IsChild(hPPUView, msg.hwnd))
+				handled = IsDialogMessage(hPPUView, &msg);
+
+			// Nametable Viewer
+			extern HWND hNTView;
+			if(!handled && hNTView && IsChild(hNTView, msg.hwnd))
+				handled = IsDialogMessage(hNTView, &msg);
+
+			// Text Hooker
+			extern HWND hTextHooker;
+			if(!handled && hTextHooker && IsChild(hTextHooker, msg.hwnd))
+				handled = IsDialogMessage(hTextHooker, &msg);
+
+			// Lua Scripts
+			extern HWND LuaConsoleHWnd;
+			if(!handled && LuaConsoleHWnd && IsChild(LuaConsoleHWnd, msg.hwnd))
+				handled = IsDialogMessage(LuaConsoleHWnd, &msg);
+
+			// Logs
+			extern HWND logwin;
+			if(!handled && logwin && IsChild(logwin, msg.hwnd))
+				handled = IsDialogMessage(logwin, &msg);
+
+			// Header Editor
+			extern HWND hHeadEditor;
+			if (!handled && hHeadEditor && IsChild(hHeadEditor, msg.hwnd))
+				handled = IsDialogMessage(hHeadEditor, &msg);
+
+			// Netplay (Though it's quite dummy and nearly forgotten)
+			extern HWND netwin;
+			if (!handled && netwin && IsChild(netwin, msg.hwnd))
+				handled = IsDialogMessage(netwin, &msg);
+
 			/* //adelikat - Currently no accel keys are used in the main window.  Uncomment this block to activate them.
 			if(!handled)
 				if(msg.hwnd == hAppWnd)
@@ -366,6 +450,7 @@ int BlockingCheck()
 					}
 				}
 			*/
+
 			if(!handled)
 			{
 				TranslateMessage(&msg);
@@ -423,28 +508,33 @@ void DoFCEUExit()
 	if (FCEUMOV_Mode(MOVIEMODE_TASEDITOR) && !exitTASEditor())
 		return;
 
-	if (CloseMemoryWatch() && AskSave())		//If user was asked to save changes in the memory watch dialog or ram watch, and chose cancel, don't close FCEUX!
+	if (CloseMemoryWatch() && AskSaveRamWatch())		//If user was asked to save changes in the memory watch dialog or ram watch, and chose cancel, don't close FCEUX!
 	{
 		if(goptions & GOO_CONFIRMEXIT)
 		{
 			//Wolfenstein 3D had cute exit messages.
-			const char * const emsg[7]={"Are you sure you want to leave?  I'll become lonely!",
+			const char * const emsg[]={
+				"Are you sure you want to leave?  I'll become lonely!",
 				"A strange game. The only winning move is not to play. How about a nice game of chess?",
 				"If you exit, I'll... EAT YOUR MOUSE.",
 				"You can never really exit, you know.",
 				"E.X.I.T?",
 				"I'm sorry, you missed your exit. There is another one in 19 miles",
 				"Silly Exit Message goes here"
-
 			};
+			const int emsg_count = sizeof(emsg) / sizeof(emsg[0]);
 
-			if(IDYES != MessageBox(hAppWnd, emsg[rand() & 6], "Exit FCE Ultra?", MB_ICONQUESTION | MB_YESNO) )
+			if(IDYES != MessageBox(hAppWnd, emsg[rand() % emsg_count], "Exit FCEUX?", MB_ICONQUESTION | MB_YESNO) )
 			{
 				return;
 			}
 		}
 
+		exiting = 1;
+
 		KillDebugger(); //mbg merge 7/19/06 added
+		ResetWatches();
+		KillMemView(); 
 
 		FCEUI_StopMovie();
 		FCEUD_AviStop();
@@ -452,7 +542,6 @@ void DoFCEUExit()
 		FCEU_LuaStop(); // kill lua script before the gui dies
 #endif
 
-		exiting = 1;
 		closeGame = true;//mbg 6/30/06 - for housekeeping purposes we need to exit after the emulation cycle finishes
 		// remember the ROM name
 		extern char LoadedRomFName[2048];
@@ -634,6 +723,15 @@ int main(int argc,char *argv[])
 	// Parse the commandline arguments
 	t = ParseArgies(argc, argv);
 
+	if(PlayInput)
+		PlayInputFile = fopen(PlayInput, "rb");
+	if(DumpInput)
+		DumpInputFile = fopen(DumpInput, "wb");
+
+	extern int disableBatteryLoading;
+	if(PlayInput || DumpInput)
+		disableBatteryLoading = 1;
+
 	int saved_pal_setting = !!pal_emulation;
 
 	if (ConfigToLoad)
@@ -656,6 +754,7 @@ int main(int argc,char *argv[])
         pauseAfterPlayback = !!pauseAfterPlayback;
         closeFinishedMovie = !!closeFinishedMovie;
         EnableBackgroundInput = !!EnableBackgroundInput;
+		dendy = !!dendy;
 
 		KeyboardSetBackgroundAccess(EnableBackgroundInput!=0);
 		JoystickSetBackgroundAccess(EnableBackgroundInput!=0);
@@ -680,9 +779,10 @@ int main(int argc,char *argv[])
 	DoVideoConfigFix();
 	DoTimingConfigFix();
 
+	//restore the last user-set palette (cpalette and cpalette_count are preserved in the config file)
 	if(eoptions & EO_CPALETTE)
 	{
-		FCEUI_SetPaletteArray(cpalette);
+		FCEUI_SetUserPalette(cpalette,cpalette_count);
 	}
 
 	if(!t)
@@ -690,8 +790,6 @@ int main(int argc,char *argv[])
 		fullscreen=0;
 	}
 
-	CreateMainWindow();
-
 	// Do single instance coding, since we now know if the user wants it,
 	// and we have a source window to send from
 	// http://wiki.github.com/ffi/ffi/windows-examples
@@ -720,23 +818,29 @@ int main(int argc,char *argv[])
 			{
 				//kill this one, activate the other one
 				SetActiveWindow(DoInstantiatedExitWindow);
+				if(IsIconic(DoInstantiatedExitWindow))
+					ShowWindow(DoInstantiatedExitWindow, SW_RESTORE); 
+				SetForegroundWindow(DoInstantiatedExitWindow);
 				do_exit();
 				return 0;
 			}
 		}
 	}
 
-	if(!InitDInput())
+	CreateMainWindow();
+
+	if (!InitDInput())
 	{
 		do_exit();
 		return 1;
 	}
 
-	if(!DriverInitialize())
+	if (!DriverInitialize())
 	{
 		do_exit();
 		return 1;
 	}
+	UpdateMenuHotkeys(FCEUMENU_MAIN);
 
 	debugSystem = new DebugSystem();
 	debugSystem->init();
@@ -754,12 +858,15 @@ int main(int argc,char *argv[])
 			LoadNewGamey(hAppWnd, 0);
 	}
 
-	if (pal_setting_specified)
-	{
-		// Force the PAL setting specified in the command line
-        pal_emulation = saved_pal_setting;
-        FCEUI_SetVidSystem(pal_emulation);
-	}
+	if (PAL && pal_setting_specified && !dendy_setting_specified)
+		dendy = 0;
+
+	if (PAL && !dendy)
+        FCEUI_SetRegion(1, pal_setting_specified);
+	else if (dendy)
+        FCEUI_SetRegion(2, dendy_setting_specified);
+	else
+        FCEUI_SetRegion(0, pal_setting_specified || dendy_setting_specified);
 
 	if(PaletteToLoad)
 	{
@@ -839,6 +946,9 @@ doloopy:
 			}
 			else skippy = 0;
 
+			if(FCEU_LuaFrameskip())
+				skippy = true;
+
 			FCEUI_Emulate(&gfx, &sound, &ssize, skippy); //emulate a single frame
 			FCEUD_Update(gfx, sound, ssize); //update displays and debug tools
 
@@ -885,6 +995,26 @@ void _updateWindow()
 	//UpdateTasEditor();	//AnS: moved to FCEUD_Update
 }
 
+void win_debuggerLoopStep()
+{
+	FCEUD_UpdateInput();
+	_updateWindow();
+
+	//question:
+	//should this go here, or in the loop?
+
+	// HACK: break when Frame Advance is pressed
+	extern bool frameAdvanceRequested;
+	extern int frameAdvance_Delay_count, frameAdvance_Delay;
+	if (frameAdvanceRequested)
+	{
+		if (frameAdvance_Delay_count == 0 || frameAdvance_Delay_count >= frameAdvance_Delay)
+			FCEUI_SetEmulationPaused(EMULATIONPAUSED_FA);
+		if (frameAdvance_Delay_count < frameAdvance_Delay)
+			frameAdvance_Delay_count++;
+	}
+}
+
 void win_debuggerLoop()
 {
 	//delay until something causes us to unpause.
@@ -892,20 +1022,8 @@ void win_debuggerLoop()
 	while(FCEUI_EmulationPaused() && !FCEUI_EmulationFrameStepped())
 	{
 		Sleep(50);
-		FCEUD_UpdateInput();
-		_updateWindow();
-		// HACK: break when Frame Advance is pressed
-		extern bool frameAdvanceRequested;
-		extern int frameAdvance_Delay_count, frameAdvance_Delay;
-		if (frameAdvanceRequested)
-		{
-			if (frameAdvance_Delay_count == 0 || frameAdvance_Delay_count >= frameAdvance_Delay)
-				FCEUI_SetEmulationPaused(EMULATIONPAUSED_FA);
-			if (frameAdvance_Delay_count < frameAdvance_Delay)
-				frameAdvance_Delay_count++;
-		}
+		win_debuggerLoopStep();
 	}
-	int zzz=9;
 }
 
 // Update the game and gamewindow with a new frame
@@ -1051,45 +1169,40 @@ void FCEUD_ToggleStatusIcon(void)
 	UpdateCheckedMenuItems();
 }
 
-char *GetRomName()
+std::string GetRomName(bool force)
 {
 	//The purpose of this function is to format the ROM name stored in LoadedRomFName
 	//And return a char array with just the name with path or extension
 	//The purpose of this function is to populate a save as dialog with the ROM name as a default filename
 	extern char LoadedRomFName[2048];	//Contains full path of ROM
 	std::string Rom;					//Will contain the formatted path
-	if(GameInfo)						//If ROM is loaded
-		{
+	if(GameInfo || force)						//If ROM is loaded
+	{
 		char drv[PATH_MAX], dir[PATH_MAX], name[PATH_MAX], ext[PATH_MAX];
 		splitpath(LoadedRomFName,drv,dir,name,ext);	//Extract components of the ROM path
 		Rom = name;						//Pull out the Name only
-		}
+	}
 	else
 		Rom = "";
-	char*mystring = (char*)malloc(2048*sizeof(char));
-	strcpy(mystring, Rom.c_str());		//Convert string to char*
 
-	return mystring;
+	return Rom;
 }
 
-char *GetRomPath()
+std::string GetRomPath(bool force)
 {
 	//The purpose of this function is to format the ROM name stored in LoadedRomFName
 	//And return a char array with just the name with path or extension
 	//The purpose of this function is to populate a save as dialog with the ROM name as a default filename
 	extern char LoadedRomFName[2048];	//Contains full path of ROM
 	std::string Rom;					//Will contain the formatted path
-	if(GameInfo)						//If ROM is loaded
-		{
+	if(GameInfo || force)						//If ROM is loaded
+	{
 		char drv[PATH_MAX], dir[PATH_MAX], name[PATH_MAX], ext[PATH_MAX];
 		splitpath(LoadedRomFName,drv,dir,name,ext);	//Extract components of the ROM path
 		Rom = drv;						//Pull out the Path only
 		Rom.append(dir);
-		}
+	}
 	else
 		Rom = "";
-	char*mystring = (char*)malloc(2048*sizeof(char));
-	strcpy(mystring, Rom.c_str());		//Convert string to char*
-
-	return mystring;
-}
+	return Rom;
+}
\ No newline at end of file
