@@ -48,8 +48,8 @@ using namespace std;
 
 #include "debuggersp.h"
 
-extern Name* lastBankNames;
-extern Name* loadedBankNames;
+extern Name* pageNames[32];
+extern int pageNumbersLoaded[32];
 extern Name* ramBankNames;
 
 // ################################## End of SP CODE ###########################
@@ -163,7 +163,9 @@ int Tracer_CheckClickingOnAnAddressOrSymbolicName(unsigned int lineNumber, bool
 	// find the "$" before sel_start
 	int i = sel_start - 1;
 	for (; i > sel_start - 6; i--)
-		if (i >= 0 && trace_str[i] == '$')
+		if (i >= 0 &&
+			(trace_str[i] == '$' && trace_str[i+3] != ':') || // allow zero page indexted, ignore bank
+			(trace_str[i] == ':' && trace_str[i+5] == ':'))
 			break;
 	if (i > sel_start - 6)
 	{
@@ -371,7 +373,7 @@ BOOL CALLBACK TracerResizingEnumWindowsProc(HWND hwnd, LPARAM lParam)
 	return TRUE;
 }
 
-BOOL CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
+INT_PTR CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
 	int i;
 	switch(uMsg)
@@ -433,6 +435,7 @@ BOOL CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			CheckDlgButton(hwndDlg, IDC_CHECK_LOG_BREAKPOINTS, (logging_options & LOG_BREAKPOINTS) ? BST_CHECKED : BST_UNCHECKED);
 			CheckDlgButton(hwndDlg, IDC_CHECK_SYMBOLIC_TRACING, (logging_options & LOG_SYMBOLIC) ? BST_CHECKED : BST_UNCHECKED);
 			CheckDlgButton(hwndDlg, IDC_CHECK_CODE_TABBING, (logging_options & LOG_CODE_TABBING) ? BST_CHECKED : BST_UNCHECKED);
+			CheckDlgButton(hwndDlg, IDC_CHECK_LOG_BANK_NUMBER, (logging_options & LOG_BANK_NUMBER) ? BST_CHECKED : BST_UNCHECKED);
 			
 			EnableWindow(GetDlgItem(hwndDlg, IDC_TRACER_LOG_SIZE), TRUE);
 			EnableWindow(GetDlgItem(hwndDlg, IDC_BTN_LOG_BROWSE), FALSE);
@@ -440,7 +443,7 @@ BOOL CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 			EnableTracerMenuItems();
 
 			// subclass editfield
-			IDC_TRACER_LOG_oldWndProc = (WNDPROC)SetWindowLong(GetDlgItem(hwndDlg, IDC_TRACER_LOG), GWL_WNDPROC, (LONG)IDC_TRACER_LOG_WndProc);
+			IDC_TRACER_LOG_oldWndProc = (WNDPROC)SetWindowLongPtr(GetDlgItem(hwndDlg, IDC_TRACER_LOG), GWLP_WNDPROC, (LONG_PTR)IDC_TRACER_LOG_WndProc);
 			break;
 		}
 		case WM_WINDOWPOSCHANGED:
@@ -576,6 +579,10 @@ BOOL CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 							logging_options ^= LOG_CODE_TABBING;
 							CheckDlgButton(hwndDlg, IDC_CHECK_CODE_TABBING, (logging_options & LOG_CODE_TABBING) ? BST_CHECKED : BST_UNCHECKED);
 							break;
+						case IDC_CHECK_LOG_BANK_NUMBER:
+							logging_options ^= LOG_BANK_NUMBER;
+							CheckDlgButton(hwndDlg, IDC_CHECK_LOG_BANK_NUMBER, (logging_options & LOG_BANK_NUMBER) ? BST_CHECKED : BST_UNCHECKED);
+							break;
 						case IDC_CHECK_LOG_NEW_INSTRUCTIONS:
 							logging_options ^= LOG_NEW_INSTRUCTIONS;
 							if(logging && (!PromptForCDLogger()))
@@ -597,7 +604,9 @@ BOOL CALLBACK TracerCallB(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
 							{
 								// Assemble the message to pause the game.  Uses the current hotkey mapping dynamically
 								strcpy(trace_str, "Pause the game (press ");
-								strcat(trace_str, GetKeyComboName(FCEUD_CommandMapping[EMUCMD_PAUSE]));
+								char *buttName = MakeButtString(&FCEUD_CommandMapping[EMUCMD_PAUSE], 0);
+								strcat(trace_str, buttName);
+								free(buttName);
 								strcat(trace_str, " key or snap the Debugger) to update this window.\r\n");
 								SetDlgItemText(hTracer, IDC_TRACER_LOG, trace_str);
 							}
@@ -707,7 +716,9 @@ void BeginLoggingSequence(void)
 		tracelogbufAddressesLog.resize(tracelogbufsize);
 		// Assemble the message to pause the game.  Uses the current hotkey mapping dynamically
 		strcat(trace_str, "Pause the game (press ");
-		strcat(trace_str, GetKeyComboName(FCEUD_CommandMapping[EMUCMD_PAUSE]));
+		char *buttName = MakeButtString(&FCEUD_CommandMapping[EMUCMD_PAUSE], 0);
+		strcat(trace_str, buttName);
+		free(buttName);
 		strcat(trace_str, " key or snap the Debugger) to update this window.\r\n");
 		SetDlgItemText(hTracer, IDC_TRACER_LOG, trace_str);
 		tracelogbufpos = tracelogbufusedsize = 0;
@@ -840,8 +851,8 @@ void FCEUD_TraceInstruction(uint8 *opcode, int size)
 				}
 				
 				replaceNames(ramBankNames, a, &tempAddressesLog);
-				replaceNames(loadedBankNames, a, &tempAddressesLog);
-				replaceNames(lastBankNames, a, &tempAddressesLog);
+				for(int i=0;i<ARRAY_SIZE(pageNames);i++)
+					replaceNames(pageNames[i], a, &tempAddressesLog);
 			}
 			strncpy(str_disassembly, a, LOG_DISASSEMBLY_MAX_LEN);
 			str_disassembly[LOG_DISASSEMBLY_MAX_LEN - 1] = 0;
@@ -852,11 +863,8 @@ void FCEUD_TraceInstruction(uint8 *opcode, int size)
 	{
 		// special case: an RTS opcode
 		// add "----------" to emphasize the end of subroutine
-		strcat(str_disassembly, " ");
-		int i = strlen(str_disassembly);
-		for (; i < (LOG_DISASSEMBLY_MAX_LEN - 2); ++i)
-			str_disassembly[i] = '-';
-		str_disassembly[i] = 0;
+		static const char* emphasize = " -------------------------------------------------------------------------------------------------------------------------";
+		strncat(str_disassembly, emphasize, LOG_DISASSEMBLY_MAX_LEN - strlen(str_disassembly) - 1);
 	}
 	// stretch the disassembly string out if we have to output other stuff.
 	if ((logging_options & (LOG_REGISTERS|LOG_PROCESSOR_STATUS)) && !(logging_options & LOG_TO_THE_LEFT))
@@ -932,7 +940,17 @@ void FCEUD_TraceInstruction(uint8 *opcode, int size)
 		strcat(str_result, " ");
 	}
 
-	sprintf(str_address, "$%04X:", addr);
+	if (logging_options & LOG_BANK_NUMBER)
+	{
+		if (addr >= 0x8000)
+			sprintf(str_address, "$%02X:%04X: ", getBank(addr), addr);
+		else
+			sprintf(str_address, "  $%04X: ", addr);
+	} else
+	{
+		sprintf(str_address, "$%04X: ", addr);
+	}
+
 	strcat(str_result, str_address);
 	strcat(str_result, str_data);
 	strcat(str_result, str_disassembly);
