@@ -86,12 +86,31 @@ static int32 DiskPtr;
 static int32 DiskSeekIRQ;
 static uint8 SelectDisk, InDisk;
 
+/* 4024(w), 4025(w), 4031(r) by dink(fbneo) */
+enum FDS_DiskBlockIDs { DSK_INIT = 0, DSK_VOLUME, DSK_FILECNT, DSK_FILEHDR, DSK_FILEDATA };
+static uint8  mapperFDS_control;    // 4025(w) control register
+static uint16 mapperFDS_filesize;	// size of file being read/written
+static uint8  mapperFDS_block;		// block-id of current block
+static uint16 mapperFDS_blockstart;	// start-address of current block
+static uint16 mapperFDS_blocklen;	// length of current block
+static uint16 mapperFDS_diskaddr;   // current address relative to blockstart
+static uint8  mapperFDS_diskaccess;	// disk needs to be accessed at least once before writing
+#define fds_disk() (diskdata[InDisk][mapperFDS_blockstart + mapperFDS_diskaddr])
+#define mapperFDS_diskinsert (InDisk != 255)
+
+
 #define DC_INC    1
 
 void FDSGI(GI h) {
-	switch (h) {
-	case GI_CLOSE: FDSClose(); break;
-	case GI_POWER: FDSInit(); break;
+	switch (h)
+	{
+		case GI_CLOSE: FDSClose(); break;
+		case GI_POWER: FDSInit(); break;
+
+		// Unhandled Cases
+		case GI_RESETM2:
+		case GI_RESETSAVE:
+			break;
 	}
 }
 
@@ -141,6 +160,14 @@ static void FDSInit(void) {
 	FDSSoundReset();
 	InDisk = 0;
 	SelectDisk = 0;
+
+	mapperFDS_control = 0;
+	mapperFDS_filesize = 0;
+	mapperFDS_block = 0;
+	mapperFDS_blockstart = 0;
+	mapperFDS_blocklen = 0;
+	mapperFDS_diskaddr = 0;
+	mapperFDS_diskaccess = 0;
 }
 
 void FCEU_FDSInsert(void)
@@ -233,17 +260,45 @@ static DECLFR(FDSRead4030) {
 }
 
 static DECLFR(FDSRead4031) {
-	static uint8 z = 0;
-	if (InDisk != 255) {
-		z = diskdata[InDisk][DiskPtr];
-		if (!fceuindbg) {
-			if (DiskPtr < 64999) DiskPtr++;
-			DiskSeekIRQ = 150;
-			X6502_IRQEnd(FCEU_IQEXT2);
+	static uint8 ret = 0;
+
+	ret = 0xff;
+	if (mapperFDS_diskinsert && mapperFDS_control & 0x04) {
+		mapperFDS_diskaccess = 1;
+
+		ret = 0;
+
+		switch (mapperFDS_block) {
+			case DSK_FILEHDR:
+				if (mapperFDS_diskaddr < mapperFDS_blocklen) {
+					ret = fds_disk();
+					switch (mapperFDS_diskaddr) {
+						case 13: mapperFDS_filesize = ret; break;
+						case 14:
+							mapperFDS_filesize |= ret << 8;
+							//char fdsfile[10];
+							//strncpy(fdsfile, (char*)&diskdata[InDisk][mapperFDS_blockstart + 3], 8);
+							//printf("Read file: %s (size: %d)\n"), fdsfile, mapperFDS_filesize);
+							break;
+					}
+					mapperFDS_diskaddr++;
+				}
+				break;
+			default:
+				if (mapperFDS_diskaddr < mapperFDS_blocklen) {
+					ret = fds_disk();
+					mapperFDS_diskaddr++;
+				}
+				break;
 		}
+
+		DiskSeekIRQ = 150;
+		X6502_IRQEnd(FCEU_IQEXT2);
 	}
-	return z;
+
+	return ret;
 }
+
 static DECLFR(FDSRead4032) {
 	uint8 ret;
 
@@ -533,33 +588,87 @@ static DECLFW(FDSWrite) {
 		break;
 	case 0x4023: break;
 	case 0x4024:
-		if ((InDisk != 255) && !(FDSRegs[5] & 0x4) && (FDSRegs[3] & 0x1)) {
-			if (DiskPtr >= 0 && DiskPtr < 65500) {
-				if (writeskip)
-					writeskip--;
-				else if (DiskPtr >= 2) {
-					DiskWritten = 1;
-					diskdata[InDisk][DiskPtr - 2] = V;
-				}
+		if (mapperFDS_diskinsert && ~mapperFDS_control & 0x04) {
+
+			if (mapperFDS_diskaccess == 0) {
+				mapperFDS_diskaccess = 1;
+				break;
 			}
+
+			switch (mapperFDS_block) {
+				case DSK_FILEHDR:
+					if (mapperFDS_diskaddr < mapperFDS_blocklen) {
+						fds_disk() = V;
+						DiskWritten = 1;
+						switch (mapperFDS_diskaddr) {
+							case 13: mapperFDS_filesize = V; break;
+							case 14:
+								mapperFDS_filesize |= V << 8;
+								//char fdsfile[10];
+								//strncpy(fdsfile, (char*)&diskdata[InDisk][mapperFDS_blockstart + 3], 8);
+								//printf("Write file: %s (size: %d)\n"), fdsfile, mapperFDS_filesize);
+								break;
+						}
+						mapperFDS_diskaddr++;
+					}
+					break;
+				default:
+					if (mapperFDS_diskaddr < mapperFDS_blocklen) {
+						fds_disk() = V;
+					DiskWritten = 1;
+						mapperFDS_diskaddr++;
+					}
+					break;
+			}
+
 		}
 		break;
 	case 0x4025:
 		X6502_IRQEnd(FCEU_IQEXT2);
-		if (InDisk != 255) {
-			if (!(V & 0x40)) {
-				if ((FDSRegs[5] & 0x40) && !(V & 0x10)) {
-					DiskSeekIRQ = 200;
-					DiskPtr -= 2;
+		if (mapperFDS_diskinsert) {
+			if (V & 0x40 && ~mapperFDS_control & 0x40) {
+				mapperFDS_diskaccess = 0;
+
+				DiskSeekIRQ = 150;
+
+				// blockstart  - address of block on disk
+				// diskaddr    - address relative to blockstart
+				// _block -> _blockID ?
+				mapperFDS_blockstart += mapperFDS_diskaddr;
+				mapperFDS_diskaddr = 0;
+
+				mapperFDS_block++;
+				if (mapperFDS_block > DSK_FILEDATA)
+					mapperFDS_block = DSK_FILEHDR;
+
+				switch (mapperFDS_block) {
+					case DSK_VOLUME:
+						mapperFDS_blocklen = 0x38;
+						break;
+					case DSK_FILECNT:
+						mapperFDS_blocklen = 0x02;
+						break;
+					case DSK_FILEHDR:
+						mapperFDS_blocklen = 0x10;
+						break;
+					case DSK_FILEDATA:		 // <blockid><filedata>
+						mapperFDS_blocklen = 0x01 + mapperFDS_filesize;
+						break;
 				}
-				if (DiskPtr < 0) DiskPtr = 0;
 			}
-			if (!(V & 0x4)) writeskip = 2;
-			if (V & 2) {
-				DiskPtr = 0; DiskSeekIRQ = 200;
+
+			if (V & 0x02) { // transfer reset
+				mapperFDS_block = DSK_INIT;
+				mapperFDS_blockstart = 0;
+				mapperFDS_blocklen = 0;
+				mapperFDS_diskaddr = 0;
+				DiskSeekIRQ = 150;
+			}
+			if (V & 0x40) { // turn on motor
+				DiskSeekIRQ = 150;
 			}
-			if (V & 0x40) DiskSeekIRQ = 200;
 		}
+		mapperFDS_control = V;
 		setmirror(((V >> 3) & 1) ^ 1);
 		break;
 	}
@@ -581,6 +690,7 @@ static int SubLoad(FCEUFILE *fp) {
 	uint8 header[16];
 	int x;
 
+	FCEU_fseek(fp, 0, SEEK_SET);
 	FCEU_fread(header, 16, 1, fp);
 
 	if (memcmp(header, "FDS\x1a", 4)) {
@@ -592,7 +702,7 @@ static int SubLoad(FCEUFILE *fp) {
 			TotalSides = t / 65500;
 			FCEU_fseek(fp, 0, SEEK_SET);
 		} else
-			return(0);
+			return 1;
 	} else
 		TotalSides = header[4];
 
@@ -602,18 +712,12 @@ static int SubLoad(FCEUFILE *fp) {
 	if (TotalSides < 1) TotalSides = 1;
 
 	for (x = 0; x < TotalSides; x++) {
-		diskdata[x] = (uint8*)FCEU_malloc(65500);
-		if (!diskdata[x]) {
-			int zol;
-			for (zol = 0; zol < x; zol++)
-				free(diskdata[zol]);
-			return 0;
-		}
+		if ((diskdata[x] = (uint8*)FCEU_malloc(65500)) == NULL) return 2;
 		FCEU_fread(diskdata[x], 1, 65500, fp);
 		md5_update(&md5, diskdata[x], 65500);
 	}
 	md5_finish(&md5, GameInfo->MD5.data);
-	return(1);
+	return 0;
 }
 
 static void PreSave(void) {
@@ -638,21 +742,37 @@ int FDSLoad(const char *name, FCEUFILE *fp) {
 	FILE *zp;
 	int x;
 
+	// try to load FDS image first
+	FreeFDSMemory();
+	int load_result = SubLoad(fp);
+	switch (load_result)
+	{
+	case 1:
+		FreeFDSMemory();
+		return LOADER_INVALID_FORMAT;
+	case 2:
+		FreeFDSMemory();
+		FCEU_PrintError("Unable to allocate memory.");
+		return LOADER_HANDLED_ERROR;
+	}
+
+	// load FDS BIOS next
 	char *fn = strdup(FCEU_MakeFName(FCEUMKF_FDSROM, 0, 0).c_str());
 
 	if (!(zp = FCEUD_UTF8fopen(fn, "rb"))) {
 		FCEU_PrintError("FDS BIOS ROM image missing: %s", FCEU_MakeFName(FCEUMKF_FDSROM, 0, 0).c_str());
 		free(fn);
-		return 0;
+		FreeFDSMemory();
+		return LOADER_HANDLED_ERROR;
 	}
-
 	free(fn);
 
 	fseek(zp, 0L, SEEK_END);
 	if (ftell(zp) != 8192) {
 		fclose(zp);
+		FreeFDSMemory();
 		FCEU_PrintError("FDS BIOS ROM image incompatible: %s", FCEU_MakeFName(FCEUMKF_FDSROM, 0, 0).c_str());
-		return 0;
+		return LOADER_HANDLED_ERROR;
 	}
 	fseek(zp, 0L, SEEK_SET);
 
@@ -677,22 +797,13 @@ int FDSLoad(const char *name, FCEUFILE *fp) {
 			free(FDSBIOS);
 		FDSBIOS = NULL;
 		fclose(zp);
+		FreeFDSMemory();
 		FCEU_PrintError("Error reading FDS BIOS ROM image.");
-		return 0;
+		return LOADER_HANDLED_ERROR;
 	}
 
 	fclose(zp);
 
-	FCEU_fseek(fp, 0, SEEK_SET);
-
-	FreeFDSMemory();
-	if (!SubLoad(fp)) {
-		if(FDSBIOS)
-			free(FDSBIOS);
-		FDSBIOS = NULL;
-		return(0);
-	}
-
 	if (!disableBatteryLoading) {
 		FCEUFILE *tp;
 		char *fn = strdup(FCEU_MakeFName(FCEUMKF_FDS, 0, 0).c_str());
@@ -704,15 +815,16 @@ int FDSLoad(const char *name, FCEUFILE *fp) {
 		}
 
 		if ((tp = FCEU_fopen(fn, 0, "rb", 0))) {
-			FCEU_printf("Disk was written. Auxillary FDS file open \"%s\".\n",fn);
+			FCEU_printf("Disk was written. Auxiliary FDS file open \"%s\".\n",fn);
 			FreeFDSMemory();
-			if (!SubLoad(tp)) {
-				FCEU_PrintError("Error reading auxillary FDS file.");
+			if (SubLoad(tp)) {
+				FCEU_PrintError("Error reading auxiliary FDS file.");
 				if(FDSBIOS)
 					free(FDSBIOS);
 				FDSBIOS = NULL;
 				free(fn);
-				return(0);
+				FreeFDSMemory();
+				return LOADER_HANDLED_ERROR;
 			}
 			FCEU_fclose(tp);
 			DiskWritten = 1;  /* For save state handling. */
@@ -749,16 +861,21 @@ int FDSLoad(const char *name, FCEUFILE *fp) {
 	AddExState(&SelectDisk, 1, 0, "SELD");
 	AddExState(&InDisk, 1, 0, "INDI");
 	AddExState(&DiskWritten, 1, 0, "DSKW");
+	AddExState(&mapperFDS_control, 1, 0, "CTRG");
+	AddExState(&mapperFDS_filesize, 2, 1, "FLSZ");
+	AddExState(&mapperFDS_block, 1, 0, "BLCK");
+	AddExState(&mapperFDS_blockstart, 2, 1, "BLKS");
+	AddExState(&mapperFDS_blocklen, 2, 1, "BLKL");
+	AddExState(&mapperFDS_diskaddr, 2, 1, "DADR");
+	AddExState(&mapperFDS_diskaccess, 1, 0, "DACC");
 
 	CHRRAMSize = 8192;
 	CHRRAM = (uint8*)FCEU_gmalloc(CHRRAMSize);
-	memset(CHRRAM, 0, CHRRAMSize);
 	SetupCartCHRMapping(0, CHRRAM, CHRRAMSize, 1);
 	AddExState(CHRRAM, CHRRAMSize, 0, "CHRR");
 
 	FDSRAMSize = 32768;
 	FDSRAM = (uint8*)FCEU_gmalloc(FDSRAMSize);
-	memset(FDSRAM, 0, FDSRAMSize);
 	SetupCartPRGMapping(1, FDSRAM, FDSRAMSize, 1);
 	AddExState(FDSRAM, FDSRAMSize, 0, "FDSR");
 
@@ -768,7 +885,7 @@ int FDSLoad(const char *name, FCEUFILE *fp) {
 
 	FCEUI_SetVidSystem(0);
 
-	return 1;
+	return LOADER_OK;
 }
 
 void FDSClose(void) {
